/**
 * @file
 * Provides GridStack Backbone.Crud.
 */

(function ($, Drupal, Backbone, _, _db) {

  'use strict';

  Drupal.gridstack = Drupal.gridstack || {};
  _.each(['admin', 'models', 'views'], function (key) {
    Drupal.gridstack[key] = Drupal.gridstack[key] || {};
  });

  /**
   * The GridStack region methods using jQuery UI.
   *
   * @todo replace with core Awesomplete post 8.9+, see #3076171.
   */
  Drupal.gridstack.regionUi = {

    loopRegions: function () {
      var me = this;

      me.autocompleteOptions = {
        source: _.bind(me.regionSource, me),
        focus: _.bind(me.onRegionFocused, me),
        search: _.bind(me.onRegionSearched, me),
        select: _.bind(me.onRegionSelected, me),
        isComposing: false
      };
    },

    initRegion: function (el, $box) {
      var me = this;

      me.onRegionInput(el);
      me.onRegionBlur(el);
    },

    onRegionInput: function (el) {
      var me = this;
      var $el = $(el);

      // @todo in case removed before our update, fail silently, not break it.
      // The rest of methods won't be executed here on.
      if ($el.autocomplete && typeof $el.autocomplete === 'function') {
        $el.autocomplete(me.autocompleteOptions);
      }
    },

    // SO 8302534.
    regionSource: function (request, response) {
      var me = this;
      var data = me.regions;
      var term = $.ui.autocomplete.escapeRegex(request.term);
      var startsWithMatcher = new RegExp('^' + term, 'i');
      var startsWith = $.grep(data, function (value) {
        return startsWithMatcher.test(value.label || value.value || value);
      });
      var containsMatcher = new RegExp(term, 'i');
      var contains = $.grep(data, function (value) {
        return $.inArray(value, startsWith) < 0 &&
          containsMatcher.test(value.label || value.value || value);
      });

      // response(startsWith.concat(contains));
      var results = $.ui.autocomplete.filter(startsWith.concat(contains), term);
      response(results.slice(0, 60));
    },

    onRegionSelected: function (e, ui) {
      var me = this;
      var value = ui.item.value.trim();

      e.target.value = value;
      me.onRegionBlurOrSelected(e, value);
      return false;
    },

    onRegionSearched: function (e) {
      var term = e.target.value;

      return term.length >= 1;
    },

    onRegionFocused: function () {
      return false;
    }

  };

  /**
   * The GridStack region methods using HTML5 autocomplete.
   *
   * @see https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist
   */
  Drupal.gridstack.regionHtml5 = {

    loopRegions: function () {
      var me = this;
      var template = document.getElementById('gridstack-regions');

      me.templateContent = template.content;
    },

    initRegion: function (el, $box) {
      var me = this;

      el = el.length ? el[0] : el;

      if ($box && typeof $box !== 'undefined') {
        me.setRegionId($box);
      }

      me.onRegionInput(el);
      me.onRegionBlur(el);
    },

    onRegionInput: function (el) {
      var me = this;
      var datalistEl = el.nextElementSibling;
      var templateContent = me.templateContent;

      // SO 15097563.
      var keyup = function (e) {
        if (me.isValid(datalistEl)) {
          while (datalistEl.children.length) {
            datalistEl.removeChild(datalistEl.firstChild);
          }

          var term = me.sanitizeRegion(el.value);
          var clonedOptions = templateContent.cloneNode(true);
          var startsWithMatcher = new RegExp('^' + term, 'i');

          var set = Array.prototype.reduce.call(clonedOptions.children, function searchFilter(frag, el) {
            if (startsWithMatcher && startsWithMatcher.test(el.textContent) && frag.children.length < 60) {
              frag.appendChild(el);
            }
            return frag;
          }, document.createDocumentFragment());

          datalistEl.appendChild(set);
        }
      };

      el.addEventListener('keyup', Drupal.debounce(keyup, 100));
    }

  };

  /**
   * The GridStack region methods using either jQuery UI or HTML5 autocomplete.
   */
  Drupal.gridstack.region = {

    regionSelector: '.form-text--region',

    autocompleteOptions: {},

    prepareRegions: function () {
      var me = this;

      me = _.extend(me, me.isHtml5 ? Drupal.gridstack.regionHtml5 : Drupal.gridstack.regionUi);
      me.loopRegions();

      $(me.regionSelector, me.$el).each(_.bind(me.regionAutocomplete, me));
    },

    regionAutocomplete: function (i, el) {
      this.initRegion(el);
    },

    getRegionInput: function ($box) {
      return $box.find('> .box__content > .ui-widget--region ' + this.regionSelector);
    },

    updateRegion: function ($box, node, type) {
      var me = this;
      var $region = me.getRegionInput($box);

      type = type || 'root';

      if (me.isRegion && $region.length && me.isValid(node.region)) {
        $region.val(node.region).attr('data-gs-region-value', node.region);

        me.updateRegionPlaceholder($box);
      }
    },

    setRegionId: function ($box) {
      var me = this;

      var $region = me.getRegionInput($box);
      if (me.isHtml5 && $region.length) {
        var id = 'region' + $box.data('gsBid');
        $region.attr('list', id);
        $region.next('datalist').attr('id', id);
      }
    },

    updateRegionPlaceholder: function ($box, hasChildren) {
      var me = this;
      var $nested = $box.find('.gridstack--nested');

      me.setRegionId($box);

      if (typeof hasChildren === 'undefined') {
        hasChildren = $nested.length && $nested.children().length ? true : false;
      }

      if ($nested.length) {
        var $region = me.getRegionInput($box);
        $region.attr('placeholder', hasChildren ? 'container' : 'region');
      }
    },

    onRegionBlur: function (el) {
      var me = this;

      $(el).on('blur.gsac', function (e) {
        var value = me.sanitizeRegion(e.target.value);
        e.target.value = value;
        me.onRegionBlurOrSelected(e, value);
        return false;
      });
    },

    onRegionBlurOrSelected: function (e, value) {
      var me = this;
      var target = e.target;
      var $root = $(target).closest('.box--root');
      var $nestedBox = $(target).closest('.is-box-nested');
      var box = me.getModel($root);

      if (me.isValidModel(box) && !_.isEmpty(value)) {
        value = me.sanitizeRegion(value);
        value = me.isMain ? value : null;

        if ($nestedBox.length) {
          var nested = box.get('nested');
          var index = $nestedBox.index();

          if (me.isValid(nested[index])) {
            nested[index].region = value;

            box.set('nested', nested, {silent: false});
            me.collection.trigger('change');
          }
        }
        else {
          box.set('region', value, {silent: false});
        }
      }
    },

    sanitizeRegion: function (value) {
      // SO 23187013.
      // value = value.replace(/[^A-Za-z]/g, '');
      value = value.replace(/[^a-z0-9áéíóúñü .,_-]/gim, '').toLowerCase().trim();
      value = value.split('-').join('_');
      value = value.split(' ').join('_');
      value = Drupal.checkPlain(value);
      return value;
    }

  };

  /**
   * The GridStack nested methods.
   */
  Drupal.gridstack.nested = _.extend(Drupal.gridstack.region, {

    onNestedRemoveMultiple: function (e, box, $box) {
      var me = this;
      var delta = $box.index();

      $box = $box.length ? $box : $($box);
      box = me.isValidModel(box) ? box : me.collection.at(delta);

      // Content box is not a model, no need to remove box model.
      me.removeWidget($box, 'nested');

      // Manually trigger change since nested are not backbone models.
      me.collection.trigger('change', box);
    },

    onNestedAddMultiple: function (e, box, $box) {
      var me = this;
      var delta = $box.index();

      box = me.isValidModel(box) ? box : me.collection.at(delta);
      me.isRendered = true;

      if (me.isValidModel(box)) {
        var el = $box.find('.gridstack:first');
        if (el.length) {
          me.addNestedWidget(box, el, true);
        }

        // Manually trigger change since nested are not backbone models.
        me.collection.trigger('change', box);
      }
    },

    // See ::addWidget(node, el, isDefault, gridStack)
    addNestedWidget: function (box, el, isDefault, data) {
      var me = this;
      var index = box.get('index');
      var gridStack = GridStack.init(me.nestedOptions, el);
      var len = gridStack.engine.nodes.length || 0;

      var addNestedItem = function (node, i) {
        node = node || box.defaults;
        var indexNested = isDefault ? len + 1 : i + 1;

        // Nested box is not a backbone model, only has similar attributes.
        node = me.getFakeModel(node, index, indexNested);
        var $box = me.addWidget(node, el, isDefault, gridStack);
        me.updateBoxNested($box, node);
      };

      if (isDefault) {
        addNestedItem();
      }
      else if (data.length) {
        gridStack.batchUpdate();

        _.each(data, function (node, i) {
          addNestedItem(node, i);
        }, this);

        gridStack.commit();
      }

      me.toggleNestedEmpty(el);

      me.collection.trigger('change', box);
      // Nested boxes are not backbone models, hook into gridstack events.
      gridStack.on('added removed', _.bind(me.onNestedAddedRemoved, me));
      return gridStack;
    },

    toggleNestedEmpty: function (el) {
      var me = this;
      var $el = $(el);
      var hasChildren = $el.children().length > 0;
      var $box = $el.closest('.box--root');

      if ($el.hasClass('gridstack--nested')) {
        $el[hasChildren ? 'addClass' : 'removeClass']('is-nested-not-empty');

        if ($box.length) {
          me.updateRegionPlaceholder($box, hasChildren);
        }
      }
    },

    onNestedAddedRemoved: function (e, items) {
      this.toggleNestedEmpty(e.target);
    },

    addNestedWidgetMultiple: function (box, i) {
      var me = this;
      var data = _.isUndefined(me.nestedNodes[i]) ? [] : me.nestedNodes[i];
      var id = box.get('id');
      var el = $('> .box[data-gs-bid="' + id + '"] .gridstack:first', me.$el);

      return el.length ? me.addNestedWidget(box, el, false, data) : null;
    },

    updateBoxNestedMultiple: function (box, $root) {
      var me = this;

      if (!me.isValidModel(box)) {
        return;
      }

      var index = box.get('index');
      var nested = [];
      var $nested = $('.gridstack:first', $root);
      var $boxes = $('> .box:not(.grid-stack-placeholder)', $nested);

      // Might be empty, even if not empty here due to sequence, or delay.
      if ($boxes.length) {
        var storedNested = box.get('nested');
        _.each($boxes, function (el, i) {
          var $el = $(el);
          var node = $el.data('_gridstack_node');
          var $region;

          if (me.isValid(node)) {
            var data = me.getFakeModel(node, index, (i + 1));
            var region = null;
            if (me.isRegion) {
              $region = $el.find(me.regionSelector);
              var value = $region.length ? $region.val() : null;
              if (storedNested.length && me.isValid(storedNested[i])) {
                region = _.isUndefined(storedNested[i].region) ? null : storedNested[i].region;
              }

              // Value is unreliable, must rely on box model data, if any.
              data.region = null;
              if (me.isMain) {
                data.region = region || value;
              }
            }

            nested.push(data);

            me.updateBoxNested($el, data);
          }
        });
      }

      box.set('nested', nested, {silent: false});
    },

    updateBoxNested: function ($box, node) {
      var me = this;

      $box = $box.length ? $box : $($box);
      var indexNested = node.indexNested || ($box.index() + 1);

      $box.attr('data-gs-bid', node.id);
      $box.attr('data-gs-index', indexNested);
      $box.addClass('is-box-nested');

      me.updateRegion($box, node, 'nested');

      $box.find('.btn').attr('data-gs-bid', node.id);
      if (!_.isUndefined(node.mid)) {
        $box.find('.btn').attr('data-gs-mid', node.mid);
      }
    }

  });

  /**
   * The GridStack crud methods.
   */
  Drupal.gridstack.crud = _.extend(Drupal.gridstack.nested, {
    events: {
      // Cannot use gsresizestop.
      resizestop: 'onResizeStop',
      gsresize: 'onResize',
      drag: 'onDrag'
    },

    onChange: function () {
      this.save();
    },

    onSave: function () {
      this.options.updateIcon = true;
      this.save();
    },

    onResize: function (e, sizer) {
      var me = this;
      var $box = $(e.target);

      me.updateBoxDimensions($box);
    },

    onDragResize: function (e) {
      var me = this;
      var $box = $(e.target);
      var box = me.getModel($box);

      // This is GridStack library selector.
      if (me.$el[0] === e.delegateTarget) {
        // Only a nested box needs manual update.
        // @todo move it to view along with the model if you can.
        if ($box.hasClass('is-box-nested')) {
          me.updateBoxDimensions($box);
        }
        else {
          var node = $box.data('_gridstack_node');
          if (me.isValidModel(box) && me.isValidNode(node)) {
            box.set('width', node.width, {silent: false});
            box.set('height', node.height, {silent: false});
          }
        }
      }

      // Manually trigger change since this event is not backbone's.
      me.collection.trigger('change', box);
    },

    onDrag: function (e) {
      _db.throttle(this.onDragResize(e), 200, this);
    },

    onResizeStop: function (e) {
      // @todo _db.throttle(this.onDragResize(e), 200, this);
      this.onDragResize(e);
    },

    onAdd: function (box) {
      this.addWidget(box, null, true);
    },

    widgetOptions: function (node, extra) {
      var me = this;
      // v0.5.2, https://github.com/gridstack/gridstack.js/issues/907
      return _.extend({
        x: node.x,
        y: node.y,
        width: node.width,
        height: node.height,
        // Without autoPosition, new widgets are prepended, not appended.
        autoPosition: me.nodeConfig.autoPosition || true,
        // Dont! The box can not get smaller than 12 columns.
        // minWidth: me.nodeConfig.minWidth || 1
        // This can get absurd to 225+, add a max.
        // minHeight: me.nodeConfig.minHeight || 1,
        maxHeight: me.nodeConfig.maxHeight || 60
      }, extra || {});
    },

    widgetHtml: function (box, el, type, isDefault, bid) {
      var me = this;
      var isNew = isDefault || false;

      if (type === 'root') {
        if (isNew) {
          me.addCollectionHint(box.collection);
        }

        var view = me.getCurrentView(box);
        if (me.isValid(view)) {
          return view.render().el.outerHTML;
        }
      }

      // Disable infinite nests.
      return Drupal.theme('gridStackBox', {
        id: bid || 0,
        isMain: me.isMain,
        isNested: me.isNested && !el.hasClass('gridstack--nested'),
        isRegion: me.isRegion,
        isHtml5: me.isHtml5
      });
    },

    addWidget: function (box, el, isDefault, gridStack) {
      var me = this;
      var node = box;
      var extra = {};
      var widget;
      var isFakeModel = false;
      var defaultWidth = me.nodeConfig.defaultMinWidth || 2;
      var bid = 0;

      el = el || me.$el;
      isDefault = isDefault || false;

      gridStack = gridStack || me.getGridStack(el);
      if (!me.isValid(gridStack)) {
        return;
      }

      // Unlike nested, the root boxes are valid backbone models.
      if (me.isValidModel(box)) {
        // @todo, not functional, yet. For future nested models.
        box.set('gid', me.$el.attr('id'), {silent: true});
        bid = box.get('id');

        node = box.attributes;
        extra.minWidth = me.isNested ? me.nodeConfig.nestedMinWidth : me.nodeConfig.rootMinWidth;

        defaultWidth = me.isNested ? 12 : defaultWidth;

        if (me.isSmallest) {
          defaultWidth = 1;

          if (me.isNested) {
            extra.minWidth = 12;
          }
        }

        if (me.column < 12 && me.column > 4) {
          extra.maxWidth = me.column;
        }

        widget = me.widgetHtml(box, el, 'root', isDefault, bid);
      }
      else {
        isFakeModel = true;
        widget = me.widgetHtml(box, el, 'nested', isDefault, bid);

        if (me.isSmallest) {
          extra.minWidth = 6;

          if (me.isNested) {
            defaultWidth = 12;
          }
        }
      }

      if (isDefault) {
        node.height = me.nodeConfig.defaultMinHeight || 2;
        node.width = defaultWidth;
      }

      if (!gridStack.willItFit(node.x, node.y, node.width, node.height, true)) {
        return;
      }

      widget = gridStack.addWidget(widget, me.widgetOptions(node, extra));
      if (!_.isUndefined(el) && _.isUndefined(widget.context)) {
        widget.context = el.length ? el[0] : el;
      }

      // The ::addWidget returns non-jquery object without length.
      var $box = $(widget);
      var $region = $box.find(me.regionSelector + ':first');

      if (isDefault && $region.length) {
        me.initRegion($region[0], $box);
      }

      if (isFakeModel) {
        // Cannot move it to model, _yet, since this is rendered later.
        me.updateBoxDimensions($box);
      }
      else {
        me.updateRegion($box, node, 'root');
      }

      // Disable movable, as too complex to handle with breakpoints for now.
      if (me.isNested && $box.parent('.gridstack--root').length) {
        gridStack.movable($box[0], false);
      }

      // Must trigger manually to apply attributes on added widgets.
      me.collection.trigger('change', box);
      return $box;
    },

    onRemove: function (box) {
      var me = this;
      var view = me.getCurrentView(box);

      if (me.isValid(view)) {
        view.remove();
        view.stopListening(box);
      }
    },

    shouldRemove: function (box) {
      return box.get('deleted') === true;
    },

    removeWidget: function ($box, context) {
      var me = this;

      $box = $box.length ? $box : $($box);
      context = context === 'undefined' ? 'root' : context;

      if (!$box.length) {
        return;
      }

      var box = me.getModel($box);
      var el = $box.closest('.gridstack');

      if (me.isValidModel(box) && context === 'root') {
        // Do not remove directly, instead set properties, and collect it later
        // to avoid potential memory leaks.
        box.set('deleted', true);
      }

      // @todo move it model if you can.
      var gridStack = me.getGridStack(el);
      if (me.isValid(gridStack)) {
        gridStack.removeWidget($box, true);
      }
    },

    onRootRemoveMultiple: function (e, box, $box) {
      var me = this;

      $box = $box.length ? $box : $($box);

      // Destroy the box including all nested instances.
      me.removeWidget($box, 'root');
    },

    updateBoxDimensions: function ($box, node) {
      Drupal.gridstack.admin.updateBoxDimensions($box, node);
    },

    updateBoxModel: function (box, i, reIndex) {
      var me = this;
      var index = (i + 1);

      box = me.isValidModel(box) ? box : me.collection.at(i);
      reIndex = reIndex || false;

      if (!me.isValidModel(box)) {
        return;
      }

      var id = box.get('id');
      var $box = me.getBoxById(id);
      var node = $box.data('_gridstack_node');
      var data = me.getParsedNode(node);

      if (reIndex) {
        data.index = index;
      }

      if (!_.isEmpty(data)) {
        _.each(data, function (value, key) {
          box.set(key, value, {silent: false});
        });
      }

      if (reIndex) {
        $box.attr('data-gs-index', index);
      }

      me.updateBoxDimensions($box, data);

      // Update nested boxes if required.
      if (me.isNested) {
        me.updateBoxNestedMultiple(box, $box);
      }

      me.collection.trigger('change', box);
    },

    updateBoxModelMultiple: function () {
      var me = this;
      var remove = [];
      var reIndex = false;

      me.collection.each(function (box) {
        if (me.shouldRemove(box)) {
          remove.push(box);
        }
      }, me);

      if (remove.length) {
        reIndex = true;
        me.collection.remove(remove);
      }

      me.collection.each(function (box, i) {
        me.updateBoxModel(box, i, reIndex);
      }, me);

      Drupal.gridstack.admin.lastBoxIndex = me.collection.length;
    },

    save: function () {
      var me = this;

      me.updateBoxModelMultiple();

      if (!_.isEmpty(me.saveCallback) && me.isRendered) {
        var o = me.options || {};

        me.saveCallback.callback(me.collection, o);
      }

      me.options.updateIcon = false;
    }

  });

})(jQuery, Drupal, Backbone, _, dBlazy);
