/**
 * @file
 * Provides GridStack admin utilities for Layout Builder modal.
 *
 * Cheaper alt than full-blown class overrides via hook_element_plugin_alter
 * aside from the fact other modules might also race overriding that class.
 */

(function ($, Drupal, _db) {

  'use strict';

  var _lbHighligtedClass = 'is-layout-builder-highlighted';
  var _lbSectionSelector = '.layout-builder__section';
  var _dataGsSection = 'data-gs-section';
  var _gsRoot = '.is-gs-lb';
  var _gsRootActivated = _gsRoot + '.' + _lbHighligtedClass;
  var _dataGsRegionSelector = '[data-gs-region]';
  var _dataGsMediaStorage = '[data-gs-media-storage]';
  var _gsColorPickerSelector = '.form-color--gs-color';
  var _mediaOverlay = '> .box__bg .media__overlay';
  var _formSubmitActive = 'form-submit--gs-active';
  var _palettes = '.form-wrapper--color-palettes';

  /**
   * Returns GeidStack section.
   *
   * @return {HTMLElement}
   *   The .layout-builder__section HTML element.
   */
  function getSection() {
    var $gsRoot = $(_gsRootActivated);
    return $gsRoot.closest(_lbSectionSelector).attr(_dataGsSection, true);
  }

  /**
   * Reacts on Media Library selection input change.
   *
   * @param {jQuery.Event} e
   *   The input value event.
   */
  function gsMlSelectionChange(e) {
    var $input = $(e.target);
    var value = $input.val();

    $('.' + _formSubmitActive).closest(_dataGsRegionSelector).find(_dataGsMediaStorage).val(value).trigger('change');
  }

  /**
   * Reacts on Media Library "Add media" mousedown button.
   *
   * @param {jQuery.Event} e
   *   The mousedown event.
   */
  function gsMlAdd(e) {
    $('.' + _formSubmitActive).removeClass(_formSubmitActive);
    $(e.target).addClass(_formSubmitActive);
  }

  /**
   * Reacts on Media Library "Remove media" mousedown button.
   *
   * @param {jQuery.Event} e
   *   The mousedown event.
   */
  function gsMLRemove(e) {
    $(e.target).closest(_dataGsRegionSelector).find(_dataGsMediaStorage).val('');
  }

  /**
   * Return RGBA of a color given an alpha.
   *
   * @param {String} color
   *   The color value.
   * @param {String} alpha
   *   The alpha value.
   *
   * @return {String}
   *   The rgba color.
   */
  function makeRgba(color, alpha) {
    return 'rgba(' + parseInt(color.slice(-6, -4), 16) + ',' + parseInt(color.slice(-4, -2), 16) + ',' + parseInt(color.slice(-2), 16) + ',' + alpha + ')';
  }

  /**
   * Add the suffix due to not output via form API when combine with prefix.
   *
   * @param {HTMLElement} $elm
   *   The HTML element.
   */
  function addSuffix($elm) {
    var suffix = '<span class="field-suffix messages messages--error visually-hidden"></span>';
    var $suffix = $elm.next('.field-suffix');

    // @todo  && $elm.hasClass('form-range--gs-opacity')
    if (!$suffix.length) {
      $elm.after(suffix);
    }
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .form-color--gs HTML element.
   */
  function gsColorPicker(i, elm) {
    var $gsSection;
    var $palettes = $(_palettes);

    $(elm).off('.gsColorPickerChange').on('change.gsColorPickerChange', function (e) {
      if ($palettes.length && $palettes.attr('open')) {
        $palettes.attr('open', false);
      }
    });

    $(elm).off('.gsColorPicker').on('input.gsColorPicker', function () {
      $gsSection = getSection();

      var $input = $(this);
      var color = $input.val();
      var property = $input.data('gsColorPicker');
      var isBg = property === 'background-color';
      var region = $input.data('gsColorRegion');
      var target = $input.data('gsTargetSelector') || false;
      var $rgba = $('[data-gs-color-picker="rgba"][data-gs-color-region="' + region + '"]');
      var $alpha = $('.form-range--gs-alpha[data-gs-color-region="' + region + '"]');
      var alpha = $alpha.length ? $alpha.val() : -1;
      var rgba = isBg ? makeRgba(color, alpha) : color;
      var skip = color === '#000000';
      var visibleColor = skip ? '' : rgba;

      if ($palettes.length && !$palettes.attr('open')) {
        $palettes.attr('open', true);
      }

      if ($gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"]');
        if ($selector.length) {
          if (isBg && $selector.find(_mediaOverlay).length) {
            visibleColor = rgba;
            if ($selector.hasClass('is-gs-overlay')) {
              target = _mediaOverlay;
            }
            else {
              target = false;
              $selector.find(_mediaOverlay).css(property, '');
            }
          }

          if (!target) {
            $selector.css(property, visibleColor);
          }
          else {
            if (target === 'a' || target === 'a:hover') {
              target = target + ':not(.gridstack__action a)';
            }
            // @todo a:hover, of course.
            $selector.find(target).css(property, visibleColor);
          }
        }
      }

      if (isBg && $rgba.length) {
        $rgba.val(skip ? '' : rgba);
      }
    });
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .form-wrapper--animations HTML element.
   */
  function gsAnimations(i, elm) {
    var $elm = $(elm);
    var $gsSection;
    var $animation = $('.form-select--gs-animation', elm);
    var values = [];

    $animation.find('option').each(function (i, item) {
      var value = $(item).val();
      values.push(value);
    });

    $elm.off('.gsAnimation').on('change.gsAnimation', '.form-select--gs-animation', function () {
      $gsSection = getSection();

      var $input = $(this);
      var value = $input.val();
      var region = $input.data('gsAnimationRegion');
      var target = $input.data('gsTargetSelector') || false;

      if (target && $gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"] ' + target);
        if ($selector.length) {
          _db.forEach(values, function (val) {
            $selector.removeClass(val + ' animated');
          });

          if (value) {
            $selector.addClass(value + ' animated');
            $selector.attr('data-animation', value);
          }
          else {
            $selector.removeClass(value + ' animated');
            $selector.removeAttr('data-animation');
          }
        }
      }
    });

    $elm.off('.gsTextAnimation').on('blur.gsTextAnimation', '.form-text--gs', function () {
      $gsSection = getSection();

      var $input = $(this);
      var value = $input.val();
      var key = $input.data('gsAnimationKey');
      var region = $input.data('gsAnimationRegion');
      var target = $input.data('gsTargetSelector') || false;

      if (target && $gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"] ' + target);
        if ($selector.length) {
          if (value === '') {
            $selector.removeAttr('data-animation-' + key);
            $selector.css('animation-' + key, '');
          }
          else {
            $selector.attr('data-animation-' + key, value);
            $selector.addClass('animated');
            $selector.css('animation-' + key, value);
          }
        }
      }
    });
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .form-range--gs-alpha HTML element.
   */
  function gsTransparency(i, elm) {
    var $elm = $(elm);
    var $gsSection;

    addSuffix($elm);

    // Only applicable to BG for now.
    var alphaChannel = function ($input) {
      $gsSection = getSection();

      var region = $input.data('gsColorRegion');
      var $color = $('[data-gs-color-picker="background-color"][data-gs-color-region="' + region + '"]');

      var alpha = $input.val();
      var color = $color.val();

      var $rgba = $('[data-gs-color-picker="rgba"][data-gs-color-region="' + region + '"]');
      var rgba = makeRgba(color, alpha);
      var $prefix = $input.prev('.field-prefix');
      var target = false;
      var property = 'background-color';
      var skip = color === '#000000';
      var visibleColor = skip ? 'transparent' : rgba;

      if ($gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"]');

        if ($selector.length) {
          if ($selector.find(_mediaOverlay).length) {
            if ($selector.hasClass('is-gs-overlay')) {
              target = _mediaOverlay;
            }
            else {
              target = false;
              // Remove pevious color, if switched.
              $selector.find(_mediaOverlay).css(property, '');
            }
          }

          if (!target) {
            $selector.css(property, visibleColor);
          }
          else {
            // @todo a:hover, of course.
            $selector.find(target).css(property, visibleColor);
          }
        }
      }

      // Store values regardless empty selectors, likely before adding images.
      if ($rgba.length) {
        $rgba.val(skip ? '' : rgba);
      }

      if ($prefix.length) {
        $prefix.text(alpha);
      }
    };

    var imageOpacity = function ($input) {
      $gsSection = getSection();

      var region = $input.data('gsColorRegion');
      var alpha = $input.val();
      var target = $input.data('gsTargetSelector') || false;
      var $prefix = $input.prev('.field-prefix');

      if (target && $gsSection.length) {
        var $selector = $gsSection.find('[data-region="' + region + '"] ' + target);
        if ($selector.length) {
          var $media = $selector.find('> .media__element');
          if ($media.length) {
            $selector.css('opacity', '');
            $selector = $media;
          }
          $selector.css('opacity', alpha);
        }
      }

      // Indicate values regardless empty, likely before adding images.
      if ($prefix.length) {
        $prefix.text(alpha);
      }
    };

    $(elm).off('.gsTransparency').on('change.gsTransparency', function (e) {
      var $input = $(e.target);
      var type = $input.data('gsColorPicker');

      if (type === 'alpha') {
        alphaChannel($input);
      }
      else if (type === 'opacity') {
        imageOpacity($input);
      }
    });
  }

  /**
   * GridStack functions.
   *
   * @param {HTMLElement} $input
   *   The input HTML element.
   * @param {Integer} value
   *   The default input value for an error.
   */
  function showError($input, value) {
    var $prefix = $input.prev('.field-prefix');
    var $suffix = $input.next('.field-suffix');

    if ($suffix.length) {
      $suffix.removeClass('visually-hidden').text(Drupal.t('Add image, or save the form first.'));
    }

    window.setTimeout(function () {
      $suffix.text('').assClass('visually-hidden');
    }, 6000);

    $input.val(value);

    if ($prefix.length) {
      $prefix.text(value);
    }
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .form-color--gs-height HTML element.
   */
  function gsMediaHeight(i, elm) {
    var $elm = $(elm);
    var $gsSection;

    addSuffix($elm);

    var adjustMediaHeight = function (e) {
      $gsSection = getSection();
      if (!$gsSection.length) {
        return;
      }

      var $input = $(e.target);
      var region = $input.data('gsColorRegion');
      var value = $input.val();
      var $prefix = $input.prev('.field-prefix');
      var target = $input.data('gsTargetSelector') || '';
      var $selector = $gsSection.find('[data-region="' + region + '"]' + target);
      if ($selector.length) {
        value = value < 100 ? '' : value;
        $selector.css('height', value + 'px');
      }
      else {
        showError($input, 0);
      }

      if ($prefix.length) {
        $prefix.text(value);
      }
    };

    $(elm).off('.gsMediaHeight').on('change.gsMediaHeight', adjustMediaHeight);
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The [data-gs-preset-region] HTML element.
   */
  function gsPreset(i, elm) {
    var $elm = $(elm);
    var values = [];
    var $gsSection;

    $elm.find('input').each(function (i, item) {
      var value = $(item).val();
      values.push(value);
    });

    $elm.off('.gsPreset').on('click.gsPreset', 'input', function (e) {
      $gsSection = getSection();
      if (!$gsSection.length) {
        return;
      }

      var input = e.target;
      var $input = $(input);
      var target = $input.data('gsTargetSelector') || '';
      var value = $input.val();
      var region = $input.attr('data-gs-preset-region');
      var $selector = $gsSection.find('[data-region="' + region + '"]' + target);

      if ($selector.length) {
        // @todo var $content = $selector.find('> .box__content');
        // @todo if ($content.length) {
        // @todo  $selector = $content;
        // @todo }
        _db.forEach(values, function (val) {
          $selector.removeClass(val);
        });

        if (value) {
          $selector.addClass(value);
        }
      }
    });
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .form-wrapper--extras HTML element.
   */
  function gsExtras(i, elm) {
    var $elm = $(elm);
    var $gsSection;

    $elm.off('.gsExtras').on('click.gsExtras', 'input', function (e) {
      $gsSection = getSection();
      if (!$gsSection.length) {
        return;
      }

      var input = e.target;
      var $input = $(input);
      var value = $input.val();
      var cssClass = input.dataset.gsExtrasRegionClass;
      var region = $input.attr('data-gs-extras-region');
      var $selector = $gsSection.find('[data-region="' + region + '"]');
      var $rgba = $('.form-color--gs-rgba[data-gs-color-region="' + region + '"]');
      var $bg = $('.form-color--gs-bg[data-gs-color-region="' + region + '"]');

      if ($selector.length) {
        var $content = $selector.find('> .box__content');
        if ($content.length) {
          $selector = $content;
        }

        var checked = $input.prop('checked');
        $selector[checked ? 'addClass' : 'removeClass'](cssClass);
        if (cssClass === 'is-gs-ete') {
          $selector.closest('.box')[checked ? 'addClass' : 'removeClass']('box--ete');
        }
        var $overlay = $selector.find(_mediaOverlay);
        switch (value) {
          case 'overlay':
            if ($overlay.length) {
              var previousColor = $rgba.val() || $bg.val();
              if (previousColor) {
                if (checked) {
                  $overlay.css('background-color', previousColor);
                }
                else {
                  $selector.css('background-color', previousColor);
                }
              }

              $overlay[checked ? 'show' : 'hide']();
            }
            break;

          default:
            break;
        }
      }
    });
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .is-collapsible HTML element.
   */
  function gsFieldsetCollapsible(i, elm) {
    var $elm = $(elm);

    $elm.find('> legend > span').addClass('btn button');
    $elm.off('.gsFieldsetCollapsible').on('click.gsFieldsetCollapsible', 'legend', function (e) {
      var $fieldset = $(e.delegateTarget);
      $fieldset.toggleClass('is-collapsed');
    });
  }

  /**
   * GridStack functions.
   *
   * @param {int} i
   *   The index of the current element.
   * @param {HTMLElement} elm
   *   The .form-item--vm HTML element.
   */
  function gsVerticalMargin(i, elm) {
    var $elm = $(elm);
    var values = [];
    var $gsRoot;

    $elm.children().each(function (i, item) {
      values.push(item.value);
    });

    $elm.off('.gsVerticalMargin').on('change.gsVerticalMargin', function (e) {
      $gsRoot = $(_gsRootActivated);
      if (!$gsRoot.length) {
        return;
      }

      var $select = $(e.target);
      var value = $select.val();

      _db.forEach(values, function (val) {
        $gsRoot.removeClass('vm-' + val);
      });
      $gsRoot.addClass('vm-' + value);
    });
  }

  /**
   * Attaches gridstack behavior to HTML element .gridstack.is-gs-lb.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.gridStackAdminModal = {
    attach: function (context) {

      // '.form-color--gs'
      $(_gsColorPickerSelector, context).once('gsColorPicker').each(gsColorPicker);
      $('.form-range--gs-color', context).once('gsTransparency').each(gsTransparency);
      $('.form-wrapper--animations', context).once('gsAnimations').each(gsAnimations);

      $('.form-range--gs-lg-height', context).once('gsMediaHeight').each(gsMediaHeight);
      $('fieldset.is-collapsible', context).once('gsFieldsetCollapsible').each(gsFieldsetCollapsible);

      $('.form-wrapper[data-gs-preset-region]:not(input)', context).once('gsPreset').each(gsPreset);
      $('.form-wrapper--extras:not(input)', context).once('gsExtras').each(gsExtras);

      $('.form-item--vm', context).once('gsVerticalMargin').each(gsVerticalMargin);

      $('.form-submit--gs-remove', context).once('gsMLRemove').on('mousedown.gsMLRemove', gsMLRemove);
      $('.form-submit--gs-add-replace', context).once('gsMlAdd').on('mousedown.gsMLAdd', gsMlAdd);
      $('#media-library-modal-selection', context).once('gsMlSelectionChange').on('change.gsMlSelectionChange', gsMlSelectionChange);

    },
    detach: function (context, settings, trigger) {
      if (trigger === 'unload') {
        if ($(context).find(_gsColorPickerSelector).removeOnce('gsColorPicker').length) {
          $(context).find(_gsColorPickerSelector).off('input.gsColorPicker');
        }
      }
    }
  };

})(jQuery, Drupal, dBlazy);
