/**
 * @file
 * Provides basic GridStack CSS rules utilities.
 */

(function (factory) {

  'use strict';

  // Browser globals (root is window).
  factory(window.dBlazy, window);

})(function (dBlazy, scope) {

  'use strict';

  var GridStyle = {

    createStylesheet: function (id, parent) {
      var style = document.createElement('style');

      style.setAttribute('type', 'text/css');
      style.setAttribute('data-gs-style-id', id);

      if (style.styleSheet) {
        style.styleSheet.cssText = '';
      }
      else {
        style.appendChild(document.createTextNode(''));
      }

      if (!parent) {
        // Default to head.
        parent = document.getElementsByTagName('head')[0];
      }

      parent.insertBefore(style, parent.firstChild);
      return style.sheet;
    },

    removeStylesheet: function (id) {
      var style = document.querySelector('STYLE[data-gs-style-id=' + id + ']');
      if (style !== null) {
        style.parentNode.removeChild(style);
      }
    },

    insertCSSRule: function (sheet, selector, rules, index) {
      if (typeof sheet.insertRule === 'function') {
        sheet.insertRule(selector + '{' + rules + '}', index);
      }
      else if (typeof sheet.addRule === 'function') {
        sheet.addRule(selector, rules, index);
      }
    },

    parseHeight: function (val) {
      var height = val;
      var heightUnit = 'px';
      if (height && typeof height === 'string') {
        var match = height.match(/^(-[0-9]+\.[0-9]+|[0-9]*\.[0-9]+|-[0-9]+|[0-9]+)(px|em|rem|vh|vw|%)?$/);
        if (!match) {
          throw new Error('Invalid height');
        }
        heightUnit = match[2] || 'px';
        height = parseFloat(match[1]);
      }
      return {height: height, unit: heightUnit};
    },

    defaults: function (target) {
      var sources = Array.prototype.slice.call(arguments, 1);
      var proto = Object.prototype;

      sources.forEach(function (source) {
        for (var prop in source) {
          if (proto.hasOwnProperty.call(source, prop) && (!proto.hasOwnProperty.call(target, prop) || typeof target[prop] === 'undefined')) {
            target[prop] = source[prop];
          }
        }
      });

      return target;
    }

  };

  scope.GridStyle = GridStyle;

  /**
   * Initializing the grid style utilities.
   *
   * @param {object} opts
   *   The GridStatic options.
   * @param {HTMLElement} el
   *   The GridStatic HTML element.
   *
   * @return {function}
   *   The GridStatic instance.
   */
  GridStyle.init = function (opts, el) {
    var me = this;
    var prefix = opts.prefix || 'gridstack--style-';

    if (opts.id) {
      GridStyle.removeStylesheet(opts.id);
    }

    opts.id = prefix + (Math.random() * 100000).toFixed();

    // Insert style to parent (instead of 'head') to support WebComponent.
    me._styles = GridStyle.createStylesheet(opts.id, el.parentNode);
    if (me._styles !== null) {
      me._styles._max = 0;
    }
    return {id: opts.id, styles: me._styles};
  };

  return scope.GridStyle;

});
