/**
 * @file
 * Provides methods for anything related to GridStack, including Outlayer.
 */

(function (Drupal, _db, window) {

  'use strict';

  var noop = function () {};

  Drupal.gridstack = Drupal.gridstack || {};

  /**
   * GridStack front-end public methods.
   *
   * @namespace
   */
  Drupal.gridstack.base = {
    $el: null,
    $instance: null,
    $items: [],
    $sizer: null,
    cellHeight: 0,
    columns: null,
    itemSelector: '.gridstack__box:not(.is-nixbox)',
    minWidth: 480,
    onClass: null,
    options: {},
    vm: 0,
    windowWidth: _db.windowWidth(),
    isBlazy: false,
    isLayoutBuilder: false,
    isNativeGrid: false,
    isGridStack: false,
    isUnGridStack: false,
    updateAttribute: noop,
    updateNodes: noop,
    preUpdateGrid: noop,
    postUpdateGrid: noop,
    fixAspectRatio: noop,
    destroy: noop,
    load: noop,

    column: function (el) {
      return el.dataset.gsColumn ? parseInt(el.dataset.gsColumn) : 0;
    },

    isEnabled: function (el) {
      // @todo var minWidth = typeof el === 'undefined' ? this.minWidth : parseInt(el.dataset.gsMinWidth);
      return this.windowWidth >= this.minWidth;
    },

    isValid: function (o) {
      return o !== null && o !== 'undefined';
    },

    isValidParam: function (o) {
      return o && typeof o !== 'undefined';
    },

    updateClasses: function (el, e) {
      var me = this;
      var enabled = me.isEnabled(el);
      var matches = el.className.match(/\d+/g);

      if (me.isValid(matches)) {
        el.className = el.className.replace(/grid-stack-(\d+)/g, '');
      }

      // At times, the library didn't set this on for some reason.
      if (me.column(el) > 0) {
        el.classList.add('grid-stack-' + me.column(el));
      }

      el.classList[enabled ? 'add' : 'remove']('is-gs-enabled');
      el.classList[!enabled ? 'add' : 'remove']('is-gs-disabled');

      if (me.options._class) {
        el.classList[enabled ? 'add' : 'remove'](me.options._class);
      }
    },

    outerHeight: function (el) {
      var style = window.getComputedStyle(el);

      return ['top', 'bottom']
        .map(function (side) {
          return parseInt(style['padding-' + side], 10);
        })
        .reduce(function (total, side) {
          return total + side;
        }, 0);
    },

    skip: function (box) {
      return box.querySelector('.grid') !== null;
    },

    updateRatio: function (box, el, e) {
      var me = this;

      // @todo remove check when one column mode is configurable.
      // See https://github.com/gridstack/gridstack.js/issues/304.
      if (me.skip(box)) {
        return;
      }

      var enabled = me.isEnabled(el);
      var cleaned = false;
      var stamp = _db.hasClass(box, 'box--rich') || _db.hasClass(box, 'box--stamp');
      var needFix = _db.hasClass(box, 'is-gs-ratio');
      var defaultHeight = stamp ? 2 : 1;
      var width = enabled ? parseInt(box.dataset.gsWidth) : 1;
      var height = enabled ? parseInt(box.dataset.gsHeight) : defaultHeight;
      var media = box.querySelectorAll('.b-noratio, .is-gs-fh');
      var content = box.querySelector('.box__content');
      var pad = Math.round(((height / width) * 100), 2);

      // This shows how problematic gapless layouts to address various contents:
      // w/o images, or just captions, or even a HTML block.
      // We might still miss a thing, but we can always work it out later.
      if ((needFix || media.length) && height > 0) {
        // Do this hack because content might be empty like blazy BG while we
        // need to catch a height. We'll remove it once done selectively as
        // needed far below.
        content.style.paddingBottom = pad + '%';
        var rectHeight = content.getBoundingClientRect().height;

        // Provides the height matching its container to not collapse.
        if (media.length && (rectHeight > me.cellHeight || me.cellHeight === 'auto')) {
          _db.forEach(media, function (medium) {
            medium.style.height = rectHeight + 'px';
            // Ensures those without this class to have it for consistent rules.
            medium.classList.add('b-noratio');
          });

          // Only cleans up if we have a defined height above, else collapsed
          // such for boxes without media or images, just captions.
          // Ungridstack means no native Grid nor GridStack engine to designate
          // box dimensions, epecially the height, as otherwise collapsed. Hence
          // why the old padding hack above is still expected.
          if (!me.isUnGridStack) {
            me.cleanUpContent(content);
            cleaned = true;
          }
        }

        // Contents are mostly blocks, with likely few embedded media/ slick.
        // Padding hack is needed for disabled without contents, else collapsed.
        // Yes, native Grid is cool without padding hack as long as enabled.
        if ((!cleaned && me.isLayoutBuilder) || (!me.isUnGridStack && enabled)) {
          me.cleanUpContent(content);
        }
      }
    },

    cleanUpContent: function (content) {
      content.style.paddingBottom = '';
      if (!content.getAttribute('style')) {
        content.removeAttribute('style');
      }
    },

    update: function (el, e) {
      var me = this;
      var slider = el.querySelector('.slick__slider');

      me.updateSizer(el, e);

      me.updateGrid(el, e);
      me.updateClasses(el, e);

      if (slider !== null && typeof e !== 'undefined') {
        slider.slick.refresh();
      }
    },

    updateSizer: function (el, e) {
      var me = this;

      if (me.isValid(me.$sizer)) {
        // Reset to calc new dimensions based on new screen width.
        me.$sizer.classList.remove('is-gs-sized');

        // Required to determine the correct aspect ratio.
        var content = me.$sizer.querySelector('.box__content');

        var fixRatio = function () {
          me.cellHeight = me.outerHeight(content);

          // @todo remove debug: me.$sizer.setAttribute('data-gs-ch', me.cellHeight);
          // Fixed aspect ratio earlier, before building layouts.
          // Runs it before init to pass the options to gridstack.
          // Let each layout engine fix their own problem.
          me.fixAspectRatio(el, e);
        };

        if (typeof e === 'undefined') {
          fixRatio();
        }
        else {
          // Delay to support resizing. Devs still resize these days.
          // How much I hate setTimeout, it always fixes issues.
          window.setTimeout(function () {
            fixRatio();
          });
        }

        // Takes the sizer out of the grid flow, and internal grid calc.
        window.setTimeout(function () {
          me.$sizer.classList.add('is-gs-sized');

          // All two dimensional, except native, has a fixed height, remove.
          if (!me.isEnabled(el)) {
            el.style.height = '';
          }
        }, 300);
      }
    },

    buildOut: function (el) {
      var me = this;

      var doResize = function (e) {
        me.windowWidth = _db.windowWidth();

        me.update(el, e);
      };

      // Two tasks: load the active grid per breakpoint, and update on resizing.
      doResize();
      // Must be faster than 100 in order to be discerned by GridStack library.
      window.addEventListener('resize', Drupal.debounce(doResize, 80), false);
    },

    initials: function (el) {
      var me = this;

      me.isBlazy = Drupal.blazy && Drupal.blazy.init !== null;
      me.isLayoutBuilder = _db.hasClass(el, 'is-gs-lb');
      me.isNativeGrid = _db.hasClass(el, 'gridstack--native');
      me.isGridStack = _db.hasClass(el, 'gridstack--gs');
      me.isUnGridStack = _db.hasClass(el, 'ungridstack');
      me.minWidth = parseInt(el.dataset.gsMinWidth) || me.minWidth;
      me.vm = me.options.verticalMargin || parseInt(el.dataset.gsVm) || 0;
      me.$items = el.querySelectorAll(me.itemSelector);
      me.$sizer = el.querySelector('.gridstack__sizer');
    },

    cleanUp: function (el) {
      var me = this;

      el.classList.remove('is-gs-destroyed');
      _db.forEach(['config', 'min-width'], function (key) {
        el.removeAttribute('data-gs-' + key);
      });

      if (me.onClass !== null) {
        el.classList.add('is-' + me.onClass + '-on');
      }

      window.setTimeout(function () {
        el.classList.remove('is-gs-packing');
      }, 500);
    }

  };

})(Drupal, dBlazy, this);
