/**
 * @file
 * Provides basic GridStack front-end utilities without drag and drop.
 */

(function (factory) {

  'use strict';

  // Browser globals (root is window).
  factory(window.dBlazy, window.GridStyle, window);

})(function (dBlazy, GridStyle, scope) {

  'use strict';

  /**
   * Private variables.
   */
  var _db = dBlazy;
  var _prefix = 'grid-stack-instance-';

  /**
   * Construct GridStatic from the given element and options.
   *
   * @param {HTMLElement} el
   *   The GridStatic HTML element.
   * @param {object} opts
   *   The GridStatic options.
   *
   * @namespace
   */
  var GridStatic = function (el, opts) {
    var me = this;

    me.destroyed = true;

    if (me.destroyed) {
      me.destroyed = false;

      me._init(el, opts);
    }

    me._updateHeightsOnResize = _db.throttle(function () {
      me.cellHeight(me.cellWidth(), false);
    }, 100);

    /**
     * Called when we are being resized.
     */
    me.onResizeHandler = function () {
      me.windowWidth = _db.windowWidth();

      me.prepareNodes();

      if (me.opts.cellHeight === 'auto') {
        me._updateHeightsOnResize();
      }

      var maxHeight = me.getMaxHeight();
      me._updateStyles(maxHeight + 10);
    };

    // Must be similar to original GridStack, 100 for consistent usage.
    window.addEventListener('resize', Drupal.debounce(me.onResizeHandler, 100));
    me.onResizeHandler();
  };

  // Cache our prototype.
  var _proto = GridStatic.prototype;
  _proto.constructor = GridStatic;

  // Prepare prototype.
  _proto.defaults = function () {
    return {
      _class: _prefix + (Math.random() * 10000).toFixed(0),
      dataset: [],
      column: 12,
      minRow: 0,
      maxRow: 0,
      itemClass: 'gridstack__box',
      placeholderClass: 'grid-stack-placeholder',
      cellHeight: 60,
      verticalMargin: 20,
      auto: true,
      minWidth: 768,
      float: false,
      rtl: 'auto',
      verticalMarginUnit: 'px',
      cellHeightUnit: 'px'
    };
  };

  // Dummy BC for cross-compatibility with original library methods.
  _proto.update = function (el, x, y, width, height) {};

  _proto.prepareNodes = function (nodes) {
    var me = this;

    nodes = nodes || me.opts.dataset;

    if (nodes) {
      me.nodes = [];
      _db.forEach(nodes, function (item) {
        var node = {};
        _db.forEach(['x', 'y', 'width', 'height'], function (pos, i) {
          node[pos] = item[i];
        });

        me.nodes.push(node);
      });
    }

    // Only query DOM if no dataset are passed above.
    if (!me.nodes.length) {
      me.items = me.el.querySelectorAll('.' + me.opts.itemClass + ':not(.' + me.opts.placeholderClass + ')');
      _db.forEach(me.items, function (item, i) {
        var node = {};
        _db.forEach(['x', 'y', 'width', 'height'], function (pos, j) {
          node[pos] = item.getAttribute('data-gs-' + pos);
        });
        me.nodes.push(node);
      });
    }

    if (me.opts.cellHeight === 'auto') {
      // Make the cell square initially.
      me.cellHeight(me.cellWidth(), true);
    }
    else {
      me.cellHeight(me.opts.cellHeight, true);
    }

    me.verticalMargin(me.opts.verticalMargin, true);
  };

  _proto.getRow = function () {
    return this.nodes.reduce(function (memo, n) { return Math.max(memo, n.y + n.height); }, 0);
  };

  _proto.getMaxHeight = function () {
    var me = this;
    var maxHeight = 0;

    me.nodes.forEach(function (n) {
      maxHeight = Math.max(maxHeight, n.y + n.height);
    });

    return maxHeight;
  };

  _proto.verticalMargin = function (val, noUpdate) {
    var me = this;

    if (typeof val === 'undefined') {
      return me.opts.verticalMargin;
    }

    var heightData = GridStyle.parseHeight(val);

    if (me.opts.verticalMarginUnit === heightData.unit && me.opts.maxRow === heightData.height) {
      return;
    }

    me.opts.verticalMarginUnit = heightData.unit;
    me.opts.verticalMargin = heightData.height;

    if (!noUpdate) {
      me._updateStyles();
    }
  };

  _proto.cellHeight = function (val, noUpdate) {
    var me = this;

    // Getter - returns the opts stored height else compute it...
    if (typeof val === 'undefined') {
      if (me.opts.cellHeight && me.opts.cellHeight !== 'auto') {
        return me.opts.cellHeight;
      }

      // Compute the height taking margin into account (each row has margin
      // other than last one).
      var o = me.el.querySelector('.' + me.opts.itemClass + ':first-child');
      var height = o.getAttribute('data-gs-height');
      var verticalMargin = me.opts.verticalMargin;

      return Math.round((o.offsetHeight - (height - 1) * verticalMargin) / height);
    }

    // Setter - updates the cellHeight value if they changed.
    var heightData = GridStyle.parseHeight(val);
    // This allows correct row calc for multiple instances.
    // @todo recheck if any issue other than resize.
    if (me.opts.cellHeightUnit === heightData.unit && me.opts.cellHeight === heightData.height) {
      return;
    }
    me.opts.cellHeightUnit = heightData.unit;
    me.opts.cellHeight = heightData.height;

    if (!noUpdate) {
      me._updateStyles();
    }
  };

  _proto.cellWidth = function () {
    var me = this;

    return Math.round(me.el.offsetWidth / me.opts.column);
  };

  _proto._initStyles = function () {
    var me = this;
    var opts = {id: me._stylesId, prefix: _prefix};

    var gridStyle = GridStyle.init(opts, me.el);

    me._stylesId = gridStyle.id;
    me._styles = gridStyle.styles;
  };

  _proto._updateStyles = function (maxHeight) {
    var me = this;
    var emptyStyles = me._styles === null || me._styles === 'undefined';

    if (emptyStyles) {
      me._initStyles();
    }

    if (emptyStyles) {
      return;
    }

    var prefix = '.' + me.opts._class + ' .' + me.opts.itemClass;
    var getHeight;

    if (typeof maxHeight === 'undefined') {
      maxHeight = me._styles._max;
    }

    me._initStyles();
    me._updateContainerHeight();

    if (!me.opts.cellHeight) {
      // The rest will be handled by CSS.
      return;
    }

    if (me._styles === null || (me._styles._max !== 0 && maxHeight <= me._styles._max)) {
      // Keep me._styles._max increasing.
      return;
    }

    if (!me.opts.verticalMargin || me.opts.cellHeightUnit === me.opts.verticalMarginUnit) {
      getHeight = function (nbRows, nbMargins) {
        return (me.opts.cellHeight * nbRows + me.opts.verticalMargin * nbMargins) +
          me.opts.cellHeightUnit;
      };
    }
    else {
      getHeight = function (nbRows, nbMargins) {
        if (!nbRows || !nbMargins) {
          return (me.opts.cellHeight * nbRows + me.opts.verticalMargin * nbMargins) +
            me.opts.cellHeightUnit;
        }

        return 'calc(' + ((me.opts.cellHeight * nbRows) + me.opts.cellHeightUnit) + ' + ' +
          ((me.opts.verticalMargin * nbMargins) + me.opts.verticalMarginUnit) + ')';
      };
    }

    if (me._styles._max === 0) {
      GridStyle.insertCSSRule(me._styles, prefix, 'min-height: ' + getHeight(1, 0) + ';', 0);
    }

    if (maxHeight > me._styles._max) {
      for (var i = me._styles._max; i < maxHeight; ++i) {
        GridStyle.insertCSSRule(me._styles,
          prefix + '[data-gs-height="' + (i + 1) + '"]',
          'height: ' + getHeight(i + 1, i) + ';',
          i
        );
        GridStyle.insertCSSRule(me._styles,
          prefix + '[data-gs-min-height="' + (i + 1) + '"]',
          'min-height: ' + getHeight(i + 1, i) + ';',
          i
        );
        GridStyle.insertCSSRule(me._styles,
          prefix + '[data-gs-max-height="' + (i + 1) + '"]',
          'max-height: ' + getHeight(i + 1, i) + ';',
          i
        );
        GridStyle.insertCSSRule(me._styles,
          prefix + '[data-gs-y="' + i + '"]',
          'top: ' + getHeight(i, i) + ';',
          i
        );
      }
      me._styles._max = maxHeight;
    }
  };

  _proto._updateContainerHeight = function () {
    var me = this;
    var row = me.getRow();

    if (row < me.opts.minRow) {
      row = me.opts.minRow;
    }

    // Check for css min height. Each row is cellHeight + verticalMargin,
    // until last one which has no margin below.
    var cssMinHeight = parseInt(me.style.getPropertyValue('min-height'));
    if (cssMinHeight > 0) {
      var verticalMargin = me.opts.verticalMargin;
      var minRow = Math.round((cssMinHeight + verticalMargin) / (me.cellHeight() + verticalMargin));
      if (row < minRow) {
        row = minRow;
      }
    }

    me.el.setAttribute('data-gs-current-row', row);

    if (!me.opts.cellHeight) {
      return;
    }

    if (!me.opts.verticalMargin) {
      me.el.style.height = (row * (me.opts.cellHeight)) + me.opts.cellHeightUnit;
    }
    else if (me.opts.cellHeightUnit === me.opts.verticalMarginUnit) {
      me.el.style.height = (row * (me.opts.cellHeight + me.opts.verticalMargin) -
        me.opts.verticalMargin) + me.opts.cellHeightUnit;
    }
    else {
      me.el.style.height = 'calc(' + ((row * (me.opts.cellHeight)) + me.opts.cellHeightUnit) +
        ' + ' + ((row * (me.opts.verticalMargin - 1)) + me.opts.verticalMarginUnit) + ')';
    }
  };

  /**
   * Set/get number of columns in the grid.
   *
   * Will attempt to update existing widgets to conform to new number of
   * columns. Requires `gridstack-extra.css` or `gridstack-extra.min.css`
   * for [2-11], else you will need to generate correct CSS
   * (see https://github.com/gridstack/gridstack.js#change-grid-columns).
   *
   * @param {int} column
   *  Integer > 0 (default 12).
   *
   * @return {int|mixed}
   *   The column if as a getter, else setting its value.
   */
  _proto.column = function (column) {
    var me = this;

    // Getter - returns the opts stored mode.
    if (typeof column === 'undefined') {
      return me.opts.column;
    }

    // Setter.
    if (me.opts.column === column) {
      return;
    }

    var oldColumn = me.opts.column;

    // If we go into 1 column mode (which happens if we're sized less than
    // minWidth unless disableOneColumnMode is on)
    // then remember the original columns so we can restore.
    if (column === 1) {
      me._prevColumn = oldColumn;
    }
    else {
      delete me._prevColumn;
    }

    me.el.classList.remove('grid-stack-' + oldColumn);
    me.el.classList.add('grid-stack-' + column);

    me.opts.column = column;
  };

  _proto._init = function (el, opts) {
    var me = this;

    me.el = el;
    me.style = window.getComputedStyle(el);
    me.items = [];
    me.nodes = [];

    me.opts = GridStyle.defaults(opts, me.defaults());
    me.opts.column = parseInt(me.el.dataset.gsColumn) || me.opts.column;
    me.opts.maxRow = parseInt(me.el.dataset.gsMaxRow) || me.opts.maxRow;

    if (me.opts.rtl === 'auto') {
      me.opts.rtl = me.style.getPropertyValue('direction') === 'rtl';
    }

    if (me.opts.rtl) {
      me.el.classList.add('grid-stack-rtl');
    }

    me.el.classList.add(me.opts._class);

    me._initStyles();
  };

  _proto.destroy = function () {
    var me = this;

    window.removeEventListener('resize', me.onResizeHandler);

    me.el.classList.remove(me.opts._class);
    delete me.el.gridstack;

    GridStyle.removeStylesheet(me._stylesId);
    me.destroyed = true;
  };

  scope.GridStatic = GridStatic;

  /**
   * Initializing the HTML element into a grid.
   *
   * Will return the grid. Calling it again will simply return the existing
   * instance (ignore any passed options).
   *
   * @param {object} opts
   *   The GridStatic options.
   * @param {HTMLElement} el
   *   The GridStatic HTML element.
   *
   * @return {function}
   *   The GridStatic instance.
   */
  GridStatic.init = function (opts, el) {
    if (!el.gridstack) {
      el.gridstack = new GridStatic(el, opts);
    }
    return el.gridstack;
  };

  return scope.GridStatic;

});
